﻿/**
* StyleCollection by Grant Skinner. April 29, 2007
* Visit www.gskinner.com/blog for documentation, updates and more free code.
*
* You may distribute this class freely, provided it is not modified in any way (including
* removing this header or changing the package path).
*
* Please contact info@gskinner.com prior to distributing modified versions of this class.
*/

package com.gskinner.styles {
	
	import fl.core.UIComponent;
	import flash.utils.Dictionary;
	import flash.utils.getQualifiedClassName;
	import flash.utils.getQualifiedSuperclassName;
	import flash.utils.getDefinitionByName
	import flash.text.TextFormat;
	import flash.events.EventDispatcher;
	import flash.events.Event;
	
	public class StyleCollection extends EventDispatcher {
		
	// static interface for accessing collections globally:
		private static var styleCollections:Object = {};
		
		public static function addStyleCollection(name:String,collection:StyleCollection):void {
			styleCollections[name] = collection;
		}
		
		public static function removeStyleCollection(name:String):void {
			delete(styleCollections[name]);
		}
		
		public static function getStyleCollection(name:String):StyleCollection {
			return styleCollections[name];
		}
		
		
	// private properties:
		protected var components:Dictionary;
		protected var styles:Object;
		protected var rendererStyles:Object;
		
	// constructor:
		public function StyleCollection(name:String=null) {
			styles = {};
			rendererStyles = {};
			components = new Dictionary(true);
			
			if (name != null) {
				addStyleCollection(name,this);
			}
		}
		
	// public methods:
		public function setStyle(style:String, value:Object):void {
			if (styles[style] === value && !(value is TextFormat)) { return; }
			styles[style] = value;
			updateSubscriberStyle(style,value);
			dispatchEvent(new Event(Event.CHANGE));
		}
		
		public function getStyle(style:String):Object {
			return styles[style];
		}
		
		public function setRendererStyle(style:String, value:Object):void {
			if (rendererStyles[style] === value && !(value is TextFormat)) { return; }
			rendererStyles[style] = value;
			updateRendererStyle(style,value);
			dispatchEvent(new Event(Event.CHANGE));
		}
		
		public function getRendererStyle(style:String):Object {
			return rendererStyles[style];
		}
		
		public function applyStyles(component:UIComponent):void {
			_applyStyles(component,getStyleDefinition(component));
		}
		
		public function applyStyle(component:UIComponent,style:String):void {
			var styleDefinition:Object = getStyleDefinition(component);
			if (styleDefinition == null || styleDefinition(style) == null || styles[style] == null) { return; }
			component.setStyle(style,styles[style]);
		}
		
		public function applyRendererStyle(component:UIComponent,style:String):void {
			if (!component.hasOwnProperty("setRendererStyle")) { return; }
			component["setRendererStyle"](style,rendererStyles[style]);
		}
		
		public function addSubscriber(component:UIComponent):void {
			components[component] = getStyleDefinition(component);
			applyStyles(component);
		}
		
		public function removeSubscriber(component:UIComponent):void {
			delete(components[component]);
		}
		
		public function addSubscribers(...components:Array):void {
			for (var i:uint=0; i<components.length; i++) {
				if (components[i] is UIComponent) {
					addSubscriber(components[i]);
				}
			}
		}
		
		public function removeSubscribers(...components:Array):void {
			for (var i:uint=0; i<components.length; i++) {
				removeSubscriber(components[i]);
			}
		}
		
		public function clone():StyleCollection {
			var styleCollection:StyleCollection = new StyleCollection();
			for (var styleName:String in rendererStyles) {
				styleCollection["setRendererStyle"](styleName,rendererStyles[styleName]);
			}
			for (styleName in styles) {
				styleCollection.setStyle(styleName,styles[styleName]);
			}
			return styleCollection;
		}
		
	// private methods:
		protected function getStyleDefinition(component:UIComponent):Object {
			var target:Class = getDefinitionByName(getQualifiedClassName(component)) as Class;
			while (true) {
				if (target.hasOwnProperty("getStyleDefinition")) {
					return target["getStyleDefinition"]();
				}
				try {
					target = component.loaderInfo.applicationDomain.getDefinition(getQualifiedSuperclassName(target)) as Class;
				} catch(e:*) {
					try {
						target = getDefinitionByName(getQualifiedSuperclassName(target)) as Class;
					} catch (e:*) {
						break;
					}
				}
				if (target == UIComponent || target == null) {
					break;
				}
			}
			return null;
		}
		
		protected function _applyStyles(component:UIComponent,styleDefinition:Object):void {
			if (component.hasOwnProperty("setRendererStyle")) {
				for (var styleName:String in rendererStyles) {
					component["setRendererStyle"](styleName,rendererStyles[styleName]);
				}
			}
			if (styleDefinition == null) { return; }
			for (styleName in styles) {
				if (styleDefinition[styleName] == null) { continue; }
				component.setStyle(styleName,styles[styleName]);
			}
		}
		
		protected function updateSubscriberStyle(style:String,value:Object):void {
			for (var o:Object in components) {
				if (components[o] != null && components[o][style] == null) { continue; }
				var component:UIComponent = o as UIComponent;
				component.setStyle(style,value);
			}
		}
		
		protected function updateRendererStyle(style:String,value:Object):void {
			for (var o:Object in components) {
				if (!o.hasOwnProperty("setRendererStyle")) { continue; }
				var component:UIComponent = o as UIComponent;
				component["setRendererStyle"](style,value);
			}
		}
		
	}
	
}